package crm

import (
	"errors"
	"hongze/fms_api/models/crm"
)

// GetSellerDepartmentAndGroupMap 获取销售部门列表-包含大组和小组
func GetSellerDepartmentListWithGroupAndTeam() (sellerList []*crm.SellerAdminWithGroupTeam, err error) {
	sellerList = make([]*crm.SellerAdminWithGroupTeam, 0)
	// 获取组别列表
	groupCond := "department_id = ?"
	groupPars := make([]interface{}, 0)
	groupPars = append(groupPars, crm.SellerDepartmentId)
	groupOB := new(crm.SysGroup)
	groupList, e := groupOB.List(groupCond, groupPars)
	if e != nil {
		err = errors.New("获取组别列表失败, Err: " + e.Error())
		return
	}
	groupMap := make(map[int]*crm.SysGroup)
	for i := range groupList {
		groupMap[groupList[i].GroupId] = groupList[i]
	}

	// 获取销售部门所有销售信息
	adminCond := "department_id = ? AND enabled = 1"
	adminPars := make([]interface{}, 0)
	adminPars = append(adminPars, crm.SellerDepartmentId)
	adminOB := new(crm.Admin)
	adminList, e := adminOB.List(adminCond, adminPars)
	if e != nil {
		err = errors.New("获取销售列表失败, Err: " + e.Error())
		return
	}

	// 销售列表
	for i := range adminList {
		v := new(crm.SellerAdminWithGroupTeam)
		v.SellerId = adminList[i].AdminId
		v.SellerName = adminList[i].RealName
		g := groupMap[adminList[i].GroupId]
		if g != nil {
			if g.ParentId > 0 {
				// 三级分组
				p := groupMap[g.ParentId]
				if p != nil {
					v.GroupId = p.GroupId
					v.GroupName = p.GroupName
					v.TeamId = g.GroupId
					v.TeamName = g.GroupName
				}
			} else {
				// 二级分组
				v.GroupId = g.GroupId
				v.GroupName = g.GroupName
			}
		}
		sellerList = append(sellerList, v)
	}
	return
}

// GetSellerTeamGroupMap 获取销售组别对应的大组信息
func GetSellerTeamGroupMap() (teamGroupMap map[int]*crm.SysGroup, err error) {
	teamGroupMap = make(map[int]*crm.SysGroup)
	groupCond := `department_id = ?`
	groupPars := make([]interface{}, 0)
	groupPars = append(groupPars, crm.SellerDepartmentId)
	groupOB := new(crm.SysGroup)
	groupList, e := groupOB.List(groupCond, groupPars)
	if e != nil {
		err = errors.New("获取组别列表失败, Err: " + e.Error())
		return
	}
	groupMap := make(map[int]*crm.SysGroup)
	for i := range groupList {
		groupMap[groupList[i].GroupId] = groupList[i]
	}
	for i := range groupList {
		// 大组对应的是自己
		if groupList[i].ParentId == 0 {
			teamGroupMap[groupList[i].GroupId] = groupList[i]
			continue
		}
		teamGroupMap[groupList[i].GroupId] = groupMap[groupList[i].ParentId]
	}
	return
}
